@@ -83,9 +83,12 @@ class Settings(Stateful):
         self.base = base
         self.voting_system = self.get_setting("voting_system", "False") == "True"
         self.logging_enabled = self.get_setting("logging_enabled", "True") == "True"
+        self.online_suggestions = (
+            self.get_setting("online_suggestions", "True") == "True"
+        )
         self.people_to_party = int(self.get_setting("people_to_party", "3"))
         self.alarm_probability = float(self.get_setting("alarm_probability", "0"))
-        self.downvotes_to_kick = int(self.get_setting("downvotes_to_kick", "3"))
+        self.downvotes_to_kick = int(self.get_setting("downvotes_to_kick", "2"))
         self.max_download_size = int(self.get_setting("max_download_size", "10"))
         self.max_playlist_items = int(self.get_setting("max_playlist_items", "10"))
         self.youtube_enabled = self.get_setting("youtube_enabled", "True") == "True"
@@ -109,6 +112,7 @@ class Settings(Stateful):
         state_dict = self.base.state_dict()
         state_dict["voting_system"] = self.voting_system
         state_dict["logging_enabled"] = self.logging_enabled
+        state_dict["online_suggestions"] = self.online_suggestions
         state_dict["people_to_party"] = self.people_to_party
         state_dict["alarm_probability"] = self.alarm_probability
         state_dict["downvotes_to_kick"] = self.downvotes_to_kick
@@ -164,7 +168,13 @@ class Settings(Stateful):
                 subprocess.call(["/usr/local/sbin/raveberry/remote_enabled"]) != 0
             )
         except FileNotFoundError:
-            logging.info("scripts not installed")
+            state_dict["system_install"] = False
+        else:
+            state_dict["system_install"] = True
+            config = configparser.ConfigParser()
+            config.read(os.path.join(settings.BASE_DIR, "config/raveberry.ini"))
+            state_dict["hotspot_configured"] = config.getboolean("Modules", "hotspot")
+            state_dict["remote_configured"] = config["Remote"]["remote_key"] != ""
 
         return state_dict
 
@@ -180,6 +190,7 @@ class Settings(Stateful):
             context["local_library"] = os.readlink(library_path)
         else:
             context["local_library"] = "/"
+        context["version"] = settings.VERSION
         return render(request, "settings.html", context)
 
     def _update_mopidy_config(self, config_file=None) -> None:
@@ -316,6 +327,13 @@ class Settings(Stateful):
         Setting.objects.filter(key="logging_enabled").update(value=enabled)
         self.logging_enabled = enabled
 
+    @option
+    def set_online_suggestions(self, request: WSGIRequest) -> None:
+        """Enables or disables the voting system based on the given value."""
+        enabled = request.POST.get("value") == "true"
+        Setting.objects.filter(key="online_suggestions").update(value=enabled)
+        self.online_suggestions = enabled
+
     @option
     def set_people_to_party(self, request: WSGIRequest) -> None:
         """Sets the amount of active clients needed to enable partymode."""
@@ -630,11 +648,16 @@ class Settings(Stateful):
         if not device:
             return HttpResponseBadRequest("No device selected")
 
-        subprocess.call(
-            f"pactl set-default-sink {device}".split(),
-            stdout=subprocess.DEVNULL,
-            env={"PULSE_SERVER": "127.0.0.1"},
-        )
+        try:
+            subprocess.run(
+                ["pactl", "set-default-sink", device],
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.PIPE,
+                env={"PULSE_SERVER": "127.0.0.1"},
+                check=True,
+            )
+        except subprocess.CalledProcessError as e:
+            return HttpResponseBadRequest(e.stderr)
         # restart mopidy to apply audio device change
         subprocess.call(["sudo", "/usr/local/sbin/raveberry/restart_mopidy"])
         return HttpResponse(
@@ -987,11 +1010,6 @@ class Settings(Stateful):
     @option
     def enable_streaming(self, _request: WSGIRequest) -> HttpResponse:
         """Enable icecast streaming."""
-        if settings.DOCKER:
-            return HttpResponseBadRequest(
-                "Choose the correct docker-compose file to control streaming"
-            )
-
         icecast_exists = False
         for line in subprocess.check_output(
             "systemctl list-unit-files --full --all".split(), universal_newlines=True
@@ -1011,10 +1029,6 @@ class Settings(Stateful):
     @option
     def disable_streaming(self, _request: WSGIRequest) -> HttpResponse:
         """Disable icecast streaming."""
-        if settings.DOCKER:
-            return HttpResponseBadRequest(
-                "Choose the correct docker-compose file to control streaming"
-            )
         subprocess.call(["sudo", "/usr/local/sbin/raveberry/disable_streaming"])
         self._update_mopidy_config()
         return HttpResponse()
@@ -1082,3 +1096,47 @@ class Settings(Stateful):
     def reboot_system(self, _request: WSGIRequest) -> None:
         """Reboots the system."""
         subprocess.call(["sudo", "/usr/local/sbin/raveberry/reboot_system"])
+
+    @option
+    def get_latest_version(self, _request: WSGIRequest) -> HttpResponse:
+        try:
+            subprocess.run(
+                "pip3 install raveberry==nonexistingversion".split(),
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.PIPE,
+                universal_newlines=True,
+                check=True,
+            )
+        except subprocess.CalledProcessError as e:
+            # parse the newest verson from pip output
+            for line in e.stderr.splitlines():
+                if "from versions" in line:
+                    versions = [re.sub(r"[^0-9.]", "", token) for token in line.split()]
+                    versions = [version for version in versions if version]
+                    latest_version = versions[-1]
+                    return HttpResponse(latest_version)
+        return HttpResponseBadRequest("Could not determine latest version.")
+
+    @option
+    def get_upgrade_config(self, _request: WSGIRequest) -> HttpResponse:
+        with open(os.path.join(settings.BASE_DIR, "config/raveberry.ini")) as f:
+            config = f.read()
+        lines = config.splitlines()
+        lines = [line for line in lines if not line.startswith("#")]
+        return HttpResponse("\n".join(lines))
+
+    @option
+    def upgrade_raveberry(self, _request: WSGIRequest) -> HttpResponse:
+        @background_thread
+        def do_upgrade() -> None:
+            subprocess.call(
+                [
+                    "sudo",
+                    "/usr/local/sbin/raveberry/upgrade_raveberry",
+                    os.path.join(settings.BASE_DIR, "config/raveberry.ini"),
+                ]
+            )
+
+        do_upgrade()
+
+        return HttpResponse("Upgrading... Look for logs in /var/www/")
