@@ -22,6 +22,12 @@ except FileNotFoundError:
         f.write(SECRET_KEY)
     print("created secret key")
 
+try:
+    with open(os.path.join(BASE_DIR, "VERSION")) as f:
+        VERSION = f.read().strip()
+except FileNotFoundError:
+    VERSION = "undefined"
+
 DEBUG = bool(os.environ.get("DJANGO_DEBUG"))
 
 DOCKER = "DOCKER" in os.environ
@@ -40,6 +46,7 @@ INSTALLED_APPS = [
     "django.contrib.sessions",
     "django.contrib.messages",
     "django.contrib.staticfiles",
+    "watson",
     "compressor",
     "sass_processor",
     "channels",
@@ -54,6 +61,7 @@ MIDDLEWARE = [
     "django.contrib.messages.middleware.MessageMiddleware",
     "django.middleware.clickjacking.XFrameOptionsMiddleware",
     "core.user_manager.SimpleMiddleware",
+    "watson.middleware.SearchContextMiddleware",
 ]
 
 ROOT_URLCONF = "main.urls"
@@ -172,6 +180,8 @@ if not os.path.exists(os.path.join(BASE_DIR, "static/admin")):
 
 # adapted for django-compressor and sass-processor
 SASS_PROCESSOR_ROOT = STATIC_ROOT
+# Change this to True if you are changing scss files.
+SASS_PROCESSOR_ENABLED = False
 
 STATICFILES_FINDERS = (
     "django.contrib.staticfiles.finders.FileSystemFinder",
