@@ -46,8 +46,13 @@ fi
 echo "*** Configuring Sound Output ***"
 cp setup/pulseaudio.service /etc/systemd/system/pulseaudio.service
 
-cp --parents /etc/pulse/default.pa $BACKUP_DIR/
-LINE="load-module module-native-protocol-tcp auth-ip-acl=127.0.0.1"
+cp --parents /etc/pulse/system.pa $BACKUP_DIR/
+LINE=$(cat <<-EOF
+	load-module module-native-protocol-tcp auth-ip-acl=127.0.0.1
+	load-module module-bluetooth-policy
+	load-module module-bluetooth-discover
+EOF
+)
 FILE="/etc/pulse/system.pa"
 grep -qxF -- "$LINE" "$FILE" || echo "$LINE" >> "$FILE"
 
