@@ -7,6 +7,7 @@ specificState = function (newState) {
 
 	$('#voting_system').prop("checked", newState.voting_system);
 	$('#logging_enabled').prop("checked", newState.logging_enabled);
+	$('#online_suggestions').prop("checked", newState.online_suggestions);
 	$('#people_to_party').val(newState.people_to_party);
 	$('#alarm_probability').val(newState.alarm_probability);
 	$('#downvotes_to_kick').val(newState.downvotes_to_kick);
@@ -51,6 +52,21 @@ specificState = function (newState) {
 	$('#wifi_protected').text(newState.wifi_protected);
 	$('#tunneling_enabled').text(newState.tunneling_enabled);
 	$('#remote_enabled').text(newState.remote_enabled);
+
+	if(!newState.system_install){
+		$('.system-install-only').addClass('is-disabled');
+		$('.system-install-only').attr('disabled-note', 'This feature is only available in a system install.');
+	} else {
+		if (!newState.hotspot_configured) {
+			console.log("no hotspot")
+			$('.hotspot-functionality').addClass('is-disabled');
+			$('.hotspot-functionality').attr('disabled-note', 'Please configure hotspot during installation to use this feature.');
+		}
+		if (!newState.remote_configured) {
+			$('.remote-functionality').addClass('is-disabled');
+			$('.remote-functionality').attr('disabled-note', 'Please configure remote during installation to use this feature.');
+		}
+	}
 }
 
 $(document).ready(function() {
@@ -68,6 +84,13 @@ $(document).ready(function() {
 			successToast('');
 		});
 	});
+	$('#online_suggestions').change(function() {
+		$.post(urls['set_online_suggestions'], {
+			value: $(this).is(":checked"),
+		}).done(function() {
+			successToast('');
+		});
+	});
 	$('#people_to_party').change(function() {
 		$.post(urls['set_people_to_party'], {
 			value: $(this).val(),
@@ -190,21 +213,20 @@ $(document).ready(function() {
 		});
 	});
 
-	$('#output_devices').on('click tap', function() {
-		$.get(urls['output_devices']).done(function(response) {
-			$('#output_devices').empty();
-			$.each(response, function(i, e) {
-				let option = $('<option/>');
-				option.text(e);
-				option.on('click tap', function(e) {
-					e.stopPropagation();
-				})
-				$('#output_devices').append(option);
+	$('#output_device').focus(function () {
+		$.get(urls['output_devices']).done(function(devices) {
+			$('#output_device').autocomplete({
+				// always show all possible output devices, regardless of current input content
+				source: function(request, response) {
+					response(devices);
+				},
+				minLength: 0,
 			});
-		})
+			$('#output_device').autocomplete("search");
+		});
 	});
 	$('#set_output_device').on('click tap', function() {
-		let selected = $("#output_devices option:selected").val();
+		let selected = $("#output_device").val();
 		$.post(urls['set_output_device'], {
 			device: selected,
 		}, function(response) {
@@ -419,4 +441,27 @@ $(document).ready(function() {
 			successToast('');
 		});
 	});
+	$('#get_latest_version').on('click tap', function() {
+		$.get(urls['get_latest_version']).done(function(response) {
+			$('#latest_version').text(response);
+		}).fail(function(response) {
+			errorToast(response.responseText);
+		});
+	});
+	$('#open_upgrade_dialog').on('click tap', function() {
+		$.get(urls['get_upgrade_config']).done(function(response) {
+			$('#upgrade_config').text(response);
+		}).fail(function(response) {
+			errorToast(response.responseText);
+		});
+		$('#upgrade_modal').modal('show');
+	});
+	$('#confirm_upgrade').on('click tap', function() {
+		$('#upgrade_modal').modal('hide');
+		$.post(urls['upgrade_raveberry']).done(function(response) {
+			successToast(response);
+		}).fail(function(response) {
+			errorToast(response.responseText);
+		});
+	});
 });
